pub struct SimulatedBinaryCrossoverArgs {
pub distribution_index: f64,
pub crossover_probability: f64,
pub variable_probability: f64,
}Expand description
Input arguments for SimulatedBinaryCrossover.
Fields
distribution_index: f64The distribution index for crossover (this is the eta_c in the paper). This directly control the spread of children. If a large value is selected, the resulting children will have a higher probability of being close to their parents; a small value generates distant offsprings.
crossover_probability: f64The probability that the parents participate in the crossover. If 1.0, the parents always participate in the crossover. If the probability is lower, then the children are the exact clones of their parents (i.e. all the variable values do not change).
variable_probability: f64The probability that a variable belonging to both parents is used in the crossover. The paper uses 0.5, meaning that each variable in a solution has a 50% chance of changing its value.
Implementations